@@ -95,18 +95,6 @@ class ServiceResource(ClassRecord):
         """
         return self.service_name.get_boto3_doc_link("ServiceResource")
 
-    @property
-    def docstring(self) -> str:
-        """
-        Class docstring.
-        """
-        return (
-            "[Show boto3 documentation]"
-            f"({self.boto3_doc_link})\n"
-            "[Show boto3-stubs documentation]"
-            f"({self.service_name.get_doc_link('service_resource')})"
-        )
-
     def get_types(self) -> set[FakeAnnotation]:
         """
         Extract type annotations for collections and sub-resources.
